@@ -6,6 +6,16 @@
 ###
 
 
+### TODO: Move this to S4Vectors (or BiocBaseUtils).
+load_package_gracefully <- function(package, ...)
+{
+    if (!requireNamespace(package, quietly=TRUE))
+        stop("Could not load package ", package, ". Is it installed?\n\n  ",
+             wmsg("Note that ", ..., " requires the ", package, " package. ",
+                  "Please install it with:"),
+             "\n\n    BiocManager::install(\"", package, "\")")
+}
+
 ### TODO: This should probably go to S4Vectors (but maybe find a better name
 ### for it first).
 seq2 <- function(to, by)
@@ -22,7 +32,7 @@ seq2 <- function(to, by)
 ### bplapply2()
 ###
 ### A simple wrapper to BiocParallel::bplapply() that falls back to lapply()
-### if 'BPPARAM' is NULL.
+### if 'length(X)' <= 1 or 'BPPARAM' is NULL or 'bpnworkers(BPPARAM)' <= 1.
 ###
 ### Also disable HDF5 file locking for the duration of the bplapply() loop,
 ### just in case the user is dealing with HDF5-backed DelayedArray objects
@@ -34,7 +44,7 @@ seq2 <- function(to, by)
 
 bplapply2 <- function(X, FUN, ..., BPPARAM=NULL)
 {
-    if (is.null(BPPARAM))
+    if (length(X) <= 1L || is.null(BPPARAM))
         return(lapply(X, FUN, ...))
     if (!requireNamespace("BiocParallel", quietly=TRUE))
         stop(wmsg("Couldn't load the BiocParallel package. Please ",
@@ -46,6 +56,10 @@ bplapply2 <- function(X, FUN, ..., BPPARAM=NULL)
     #    Sys.setenv(HDF5_USE_FILE_LOCKING="FALSE")
     #    on.exit(Sys.unsetenv("HDF5_USE_FILE_LOCKING"))
     #}
+    if (!is(BPPARAM, "BiocParallelParam"))
+        stop(wmsg("'BPPARAM' must be a BiocParallelParam derivative"))
+    if (BiocParallel::bpnworkers(BPPARAM) <= 1L)
+        return(lapply(X, FUN, ...))
     BiocParallel::bplapply(X, FUN, ..., BPPARAM=BPPARAM)
 }
 
